@@ -16,23 +16,22 @@ module AbsSyn (
   RECtx(..),
   RExp(..), nullable,
   DFA(..), State(..), SNum, StartCode, Accept(..),
-  RightContext(..), showRCtx, strtype,
+  RightContext(..), showRCtx,
   encodeStartCodes, extractActions,
   Target(..),
   UsesPreds(..), usesPreds,
   StrType(..)
   ) where
 
-import CharSet ( CharSet, Encoding )
-import Map ( Map )
-import qualified Map hiding ( Map )
-import Data.IntMap (IntMap)
-import Sort ( nub' )
-import Util ( str, nl )
+import CharSet     ( CharSet, Encoding )
+import Data.Maybe  ( fromJust )
+import Data.Map    ( Map )
+import Data.IntMap ( IntMap )
+import Util        ( str, nl )
+import qualified Data.Map as Map
+import qualified Data.Set as Set
 
-import Data.Maybe ( fromJust )
-
-infixl 4 :|
+infixl 4 :||
 infixl 5 :%%
 
 -- -----------------------------------------------------------------------------
@@ -49,44 +48,53 @@ data Directive
    | TokenType String
    deriving Show
 
-data StrType = Str | Lazy | Strict
+data StrType = Str | Lazy | Strict | StrictText
+  deriving Eq
 
 instance Show StrType where
   show Str = "String"
   show Lazy = "ByteString.ByteString"
   show Strict = "ByteString.ByteString"
+  show StrictText = "Data.Text.Text"
 
 data Scheme
   = Default { defaultTypeInfo :: Maybe (Maybe String, String) }
   | GScan { gscanTypeInfo :: Maybe (Maybe String, String) }
   | Basic { basicStrType :: StrType,
             basicTypeInfo :: Maybe (Maybe String, String) }
-  | Posn { posnByteString :: Bool,
+  | Posn { posnStrType :: StrType,
            posnTypeInfo :: Maybe (Maybe String, String) }
-  | Monad { monadByteString :: Bool, monadUserState :: Bool,
+  | Monad { monadStrType :: StrType,
+            monadUserState :: Bool,
             monadTypeInfo :: Maybe (Maybe String, String) }
 
-strtype :: Bool -> String
-strtype True = "ByteString.ByteString"
-strtype False = "String"
-
-
 wrapperCppDefs :: Scheme -> Maybe [String]
 wrapperCppDefs Default {} = Nothing
 wrapperCppDefs GScan {} = Just ["ALEX_GSCAN"]
 wrapperCppDefs Basic { basicStrType = Str } = Just ["ALEX_BASIC"]
 wrapperCppDefs Basic { basicStrType = Lazy } = Just ["ALEX_BASIC_BYTESTRING"]
 wrapperCppDefs Basic { basicStrType = Strict } = Just ["ALEX_STRICT_BYTESTRING"]
-wrapperCppDefs Posn { posnByteString = False } = Just ["ALEX_POSN"]
-wrapperCppDefs Posn { posnByteString = True } = Just ["ALEX_POSN_BYTESTRING"]
-wrapperCppDefs Monad { monadByteString = False,
+wrapperCppDefs Basic { basicStrType = StrictText } = Just ["ALEX_STRICT_TEXT"]
+wrapperCppDefs Posn { posnStrType = Str } = Just ["ALEX_POSN"]
+wrapperCppDefs Posn { posnStrType = Lazy } = Just ["ALEX_POSN_BYTESTRING"]
+wrapperCppDefs Posn { posnStrType = Strict } = Just ["ALEX_POSN_BYTESTRING"]
+wrapperCppDefs Posn { posnStrType = StrictText } = Just ["ALEX_POSN_STRICT_TEXT"]
+wrapperCppDefs Monad { monadStrType = Str,
                        monadUserState = False } = Just ["ALEX_MONAD"]
-wrapperCppDefs Monad { monadByteString = True,
+wrapperCppDefs Monad { monadStrType = Strict,
                        monadUserState = False } = Just ["ALEX_MONAD_BYTESTRING"]
-wrapperCppDefs Monad { monadByteString = False,
+wrapperCppDefs Monad { monadStrType = Lazy,
+                       monadUserState = False } = Just ["ALEX_MONAD_BYTESTRING"]
+wrapperCppDefs Monad { monadStrType = StrictText,
+                       monadUserState = False } = Just ["ALEX_MONAD_STRICT_TEXT"]
+wrapperCppDefs Monad { monadStrType = Str,
                        monadUserState = True } = Just ["ALEX_MONAD", "ALEX_MONAD_USER_STATE"]
-wrapperCppDefs Monad { monadByteString = True,
+wrapperCppDefs Monad { monadStrType = Strict,
                        monadUserState = True } = Just ["ALEX_MONAD_BYTESTRING", "ALEX_MONAD_USER_STATE"]
+wrapperCppDefs Monad { monadStrType = Lazy,
+                       monadUserState = True } = Just ["ALEX_MONAD_BYTESTRING", "ALEX_MONAD_USER_STATE"]
+wrapperCppDefs Monad { monadStrType = StrictText,
+                       monadUserState = True } = Just ["ALEX_MONAD_STRICT_TEXT", "ALEX_MONAD_USER_STATE"]
 
 -- TODO: update this comment
 --
@@ -191,7 +199,7 @@ usesPreds dfa
 -- `RExp' provides an abstract syntax for regular expressions.  `Eps' will
 -- match empty strings; `Ch p' matches strings containing a single character
 -- `c' if `p c' is true; `re1 :%% re2' matches a string if `re1' matches one of
--- its prefixes and `re2' matches the rest; `re1 :| re2' matches a string if
+-- its prefixes and `re2' matches the rest; `re1 :|| re2' matches a string if
 -- `re1' or `re2' matches it; `Star re', `Plus re' and `Ques re' can be
 -- expressed in terms of the other operators.  See the definitions of `ARexp'
 -- for a formal definition of the semantics of these operators.
@@ -200,7 +208,7 @@ data RExp
   = Eps            -- ^ Empty.
   | Ch CharSet     -- ^ Singleton.
   | RExp :%% RExp  -- ^ Sequence.
-  | RExp :| RExp   -- ^ Alternative.
+  | RExp :|| RExp   -- ^ Alternative.
   | Star RExp      -- ^ Zero or more repetitions.
   | Plus RExp      -- ^ One  or more repetitions.
   | Ques RExp      -- ^ Zero or one  repetitions.
@@ -209,7 +217,7 @@ instance Show RExp where
   showsPrec _ Eps = showString "()"
   showsPrec _ (Ch _) = showString "[..]"
   showsPrec _ (l :%% r)  = shows l . shows r
-  showsPrec _ (l :| r)  = shows l . ('|':) . shows r
+  showsPrec _ (l :|| r)  = shows l . ('|':) . shows r
   showsPrec _ (Star r) = shows r . ('*':)
   showsPrec _ (Plus r) = shows r . ('+':)
   showsPrec _ (Ques r) = shows r . ('?':)
@@ -219,7 +227,7 @@ nullable :: RExp -> Bool
 nullable Eps       = True
 nullable Ch{}      = False
 nullable (l :%% r) = nullable l && nullable r
-nullable (l :|  r) = nullable l || nullable r
+nullable (l :||  r) = nullable l || nullable r
 nullable Star{}    = True
 nullable (Plus r)  = nullable r
 nullable Ques{}    = True
@@ -255,7 +263,7 @@ arexp:: RExp -> ARexp
 arexp Eps = eps_ar
 arexp (Ch p) = ch_ar p
 arexp (re :%% re') = arexp re `seq_ar` arexp re'
-arexp (re :| re') = arexp re `bar_ar` arexp re'
+arexp (re :|| re') = arexp re `bar_ar` arexp re'
 arexp (Star re) = star_ar (arexp re)
 arexp (Plus re) = plus_ar (arexp re)
 arexp (Ques re) = ques_ar (arexp re)
@@ -321,10 +329,14 @@ encodeStartCodes scan = (scan', 0 : map snd name_code_pairs, sc_hdr)
 
         code_map = Map.fromList name_code_pairs
 
-        name_code_pairs = zip (nub' (<=) nms) [1..]
+        name_code_pairs = zip nms [1..]
 
-        nms = [nm | RECtx{reCtxStartCodes = scs} <- scannerTokens scan,
-                    (nm,_) <- scs, nm /= "0"]
+        nms = Set.toAscList . Set.fromList $
+                [ nm
+                | RECtx{ reCtxStartCodes = scs } <- scannerTokens scan
+                , (nm, _) <- scs
+                , nm /= "0"
+                ]
 
 
 -- Grab the code fragments for the token actions, and replace them
@@ -369,25 +381,25 @@ extractActions scheme scanner = (scanner{scannerTokens = new_tokens}, decl_str .
             basicTypeInfo = Just (Just tyclasses, tokenty) } -> nl .
       str fun . str " :: (" . str tyclasses . str ") => " .
       str (show strty) . str " -> " . str tokenty . nl
-    Posn { posnByteString = isByteString,
+    Posn { posnStrType = strty,
            posnTypeInfo = Just (Nothing, tokenty) } -> nl .
-      str fun . str " :: AlexPosn -> " . str (strtype isByteString) . str " -> "
+      str fun . str " :: AlexPosn -> " . str (show strty) . str " -> "
       . str tokenty . nl
-    Posn { posnByteString = isByteString,
+    Posn { posnStrType = strty,
            posnTypeInfo = Just (Just tyclasses, tokenty) } -> nl .
       str fun . str " :: (" . str tyclasses . str ") => AlexPosn -> " .
-      str (strtype isByteString) . str " -> " . str tokenty . nl
-    Monad { monadByteString = isByteString,
+      str (show strty) . str " -> " . str tokenty . nl
+    Monad { monadStrType = strty,
             monadTypeInfo = Just (Nothing, tokenty) } -> nl .
       let
-        actintty = if isByteString then "Int64" else "Int"
+        actintty = if strty == Lazy then "Int64" else "Int"
       in
         str fun . str " :: AlexInput -> " . str actintty . str " -> Alex ("
       . str tokenty . str ")" . nl
-    Monad { monadByteString = isByteString,
+    Monad { monadStrType = strty,
             monadTypeInfo = Just (Just tyclasses, tokenty) } -> nl .
       let
-        actintty = if isByteString then "Int64" else "Int"
+        actintty = if strty == Lazy then "Int64" else "Int"
       in
         str fun . str " :: (" . str tyclasses . str ") =>"
       . str " AlexInput -> " . str actintty
