@@ -13,7 +13,7 @@ module Output (outputDFA) where
 import AbsSyn
 import CharSet
 import Util
-import qualified Map
+import qualified Data.Map as Map
 import qualified Data.IntMap as IntMap
 
 import Control.Monad.ST ( ST, runST )
@@ -24,7 +24,8 @@ import Data.Array.Unboxed ( UArray, elems, (!), array, listArray )
 import Data.Maybe (isJust)
 import Data.Bits
 import Data.Char ( ord, chr )
-import Data.List ( maximumBy, sortBy, groupBy, mapAccumR )
+import Data.List ( maximumBy, sortBy, mapAccumR )
+import qualified Data.List.NonEmpty as List1
 
 -- -----------------------------------------------------------------------------
 -- Printing the output
@@ -122,28 +123,28 @@ outputDFA target _ _ scheme dfa
             . str ") => Array Int ("
             . str (show strty) . str " -> " . str toktype
             . str ")\n"
-          Posn { posnByteString = isByteString,
+          Posn { posnStrType = strty,
                  posnTypeInfo = Just (Nothing, toktype) } ->
               str actions_nm . str " :: Array Int (AlexPosn -> "
-            . str (strtype isByteString) . str " -> " . str toktype
+            . str (show strty) . str " -> " . str toktype
             . str ")\n"
-          Posn { posnByteString = isByteString,
+          Posn { posnStrType = strty,
                  posnTypeInfo = Just (Just tyclasses, toktype) } ->
               str actions_nm . str " :: (" . str tyclasses
             . str ") => Array Int (AlexPosn -> "
-            . str (strtype isByteString) . str " -> " . str toktype
+            . str (show strty) . str " -> " . str toktype
             . str ")\n"
-          Monad { monadByteString = isByteString,
+          Monad { monadStrType = strty,
                   monadTypeInfo = Just (Nothing, toktype) } ->
             let
-              actintty = if isByteString then "Int64" else "Int"
+              actintty = if strty == Lazy then "Int64" else "Int"
             in
               str actions_nm . str " :: Array Int (AlexInput -> "
             . str actintty . str " -> Alex(" . str toktype . str "))\n"
-          Monad { monadByteString = isByteString,
+          Monad { monadStrType = strty,
                   monadTypeInfo = Just (Just tyclasses, toktype) } ->
             let
-              actintty = if isByteString then "Int64" else "Int"
+              actintty = if strty == Lazy then "Int64" else "Int"
             in
               str actions_nm . str " :: (" . str tyclasses
             . str ") => Array Int (AlexInput -> "
@@ -212,31 +213,31 @@ outputDFA target _ _ scheme dfa
             . str "alexScan :: (" . str tyclasses
             . str ") => AlexInput -> Int -> AlexReturn ("
             . str (show strty) . str " -> " . str toktype . str ")\n"
-          Posn { posnByteString = isByteString,
+          Posn { posnStrType = strty,
                  posnTypeInfo = Just (Nothing, toktype) } ->
               str "alex_scan_tkn :: () -> AlexInput -> " . str intty
             . str " -> " . str "AlexInput -> " . str intty
             . str " -> AlexLastAcc -> (AlexLastAcc, AlexInput)\n"
             . str "alexScanUser :: () -> AlexInput -> Int -> AlexReturn (AlexPosn -> "
-            . str (strtype isByteString) . str " -> " . str toktype . str ")\n"
+            . str (show strty) . str " -> " . str toktype . str ")\n"
             . str "alexScan :: AlexInput -> Int -> AlexReturn (AlexPosn -> "
-            . str (strtype isByteString) . str " -> " . str toktype . str ")\n"
-          Posn { posnByteString = isByteString,
+            . str (show strty) . str " -> " . str toktype . str ")\n"
+          Posn { posnStrType = strty,
                  posnTypeInfo = Just (Just tyclasses, toktype) } ->
               str "alex_scan_tkn :: () -> AlexInput -> " . str intty
             . str " -> " . str "AlexInput -> " . str intty
             . str " -> AlexLastAcc -> (AlexLastAcc, AlexInput)\n"
             . str "alexScanUser :: (" . str tyclasses
             . str ") => () -> AlexInput -> Int -> AlexReturn (AlexPosn -> "
-            . str (strtype isByteString) . str " -> " . str toktype . str ")\n"
+            . str (show strty) . str " -> " . str toktype . str ")\n"
             . str "alexScan :: (" . str tyclasses
             . str ") => AlexInput -> Int -> AlexReturn (AlexPosn -> "
-            . str (strtype isByteString) . str " -> " . str toktype . str ")\n"
+            . str (show strty) . str " -> " . str toktype . str ")\n"
           Monad { monadTypeInfo = Just (Nothing, toktype),
-                  monadByteString = isByteString,
+                  monadStrType = strty,
                   monadUserState = userState } ->
             let
-              actintty = if isByteString then "Int64" else "Int"
+              actintty = if strty == Lazy then "Int64" else "Int"
               userStateTy | userState = "AlexUserState"
                           | otherwise = "()"
             in
@@ -253,10 +254,10 @@ outputDFA target _ _ scheme dfa
             . str " -> Alex (" . str toktype . str "))\n"
             . str "alexMonadScan :: Alex (" . str toktype . str ")\n"
           Monad { monadTypeInfo = Just (Just tyclasses, toktype),
-                  monadByteString = isByteString,
+                  monadStrType = strty,
                   monadUserState = userState } ->
             let
-              actintty = if isByteString then "Int64" else "Int"
+              actintty = if strty == Lazy then "Int64" else "Int"
               userStateTy | userState = "AlexUserState"
                           | otherwise = "()"
             in
@@ -399,10 +400,10 @@ mkTables dfa = -- trace (show (defaults)) $
         best_default :: [(Int,SNum)] -> SNum
         best_default prod_list
            | null sorted = -1
-           | otherwise   = snd (head (maximumBy lengths eq))
+           | otherwise   = snd (List1.head (maximumBy lengths eq))
            where sorted  = sortBy compareSnds prod_list
                  compareSnds (_,a) (_,b) = compare a b
-                 eq = groupBy (\(_,a) (_,b) -> a == b) sorted
+                 eq = List1.groupBy (\(_,a) (_,b) -> a == b) sorted
                  lengths  a b = length a `compare` length b
 
         -- remove all the default productions from the DFA
@@ -493,18 +494,19 @@ findFreeOffset :: Int
                -> [(Int, Int)]
                -> ST s Int
 findFreeOffset off check off_arr state = do
-    -- offset 0 isn't allowed
+
+  -- offset 0 isn't allowed
   if off == 0 then try_next else do
 
     -- don't use an offset we've used before
-  b <- readArray off_arr off
-  if b /= 0 then try_next else do
+    b <- readArray off_arr off
+    if b /= 0 then try_next else do
 
-    -- check whether the actions for this state fit in the table
-  ok <- fits off state check
-  if ok then return off else try_next
- where
-        try_next = findFreeOffset (off+1) check off_arr state
+      -- check whether the actions for this state fit in the table
+      ok <- fits off state check
+      if ok then return off else try_next
+  where
+    try_next = findFreeOffset (off+1) check off_arr state
 
 -- This is an inner loop, so we use some strictness hacks, and avoid
 -- array bounds checks (unsafeRead instead of readArray) to speed
